# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from thamos.swagger_client.api_client import ApiClient


class ProvenanceApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_provenance_python(self, analysis_id, **kwargs):  # noqa: E501
        """Retrieve a provenance check result.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_provenance_python(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisResultResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_provenance_python_with_http_info(analysis_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_provenance_python_with_http_info(analysis_id, **kwargs)  # noqa: E501
            return data

    def get_provenance_python_with_http_info(self, analysis_id, **kwargs):  # noqa: E501
        """Retrieve a provenance check result.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_provenance_python_with_http_info(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisResultResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['analysis_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_provenance_python" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'analysis_id' is set
        if ('analysis_id' not in params or
                params['analysis_id'] is None):
            raise ValueError("Missing the required parameter `analysis_id` when calling `get_provenance_python`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'analysis_id' in params:
            path_params['analysis_id'] = params['analysis_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/provenance/python/{analysis_id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisResultResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_provenance_python_log(self, analysis_id, **kwargs):  # noqa: E501
        """Show logs of a provenance checks.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_provenance_python_log(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisLogResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_provenance_python_log_with_http_info(analysis_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_provenance_python_log_with_http_info(analysis_id, **kwargs)  # noqa: E501
            return data

    def get_provenance_python_log_with_http_info(self, analysis_id, **kwargs):  # noqa: E501
        """Show logs of a provenance checks.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_provenance_python_log_with_http_info(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisLogResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['analysis_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_provenance_python_log" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'analysis_id' is set
        if ('analysis_id' not in params or
                params['analysis_id'] is None):
            raise ValueError("Missing the required parameter `analysis_id` when calling `get_provenance_python_log`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'analysis_id' in params:
            path_params['analysis_id'] = params['analysis_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/provenance/python/{analysis_id}/log', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisLogResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_provenance_python_status(self, analysis_id, **kwargs):  # noqa: E501
        """Show status of a provenance check.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_provenance_python_status(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested provenance check. (required)
        :return: AnalysisStatusResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_provenance_python_status_with_http_info(analysis_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_provenance_python_status_with_http_info(analysis_id, **kwargs)  # noqa: E501
            return data

    def get_provenance_python_status_with_http_info(self, analysis_id, **kwargs):  # noqa: E501
        """Show status of a provenance check.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_provenance_python_status_with_http_info(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested provenance check. (required)
        :return: AnalysisStatusResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['analysis_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_provenance_python_status" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'analysis_id' is set
        if ('analysis_id' not in params or
                params['analysis_id'] is None):
            raise ValueError("Missing the required parameter `analysis_id` when calling `get_provenance_python_status`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'analysis_id' in params:
            path_params['analysis_id'] = params['analysis_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/provenance/python/{analysis_id}/status', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisStatusResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def post_provenance_python(self, body, **kwargs):  # noqa: E501
        """Check provenance of packages stated in an application stack.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_provenance_python(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PythonStack body: Pipfile and Pipfile.lock as used by pipenv. (required)
        :param str origin: A repository where the application stack is used. This is used for tracking as well as for automated reporting when results are available. 
        :param bool debug: Run the provenance checker in a verbose mode so developers can debug it. 
        :param bool force: Do not use cached results, always run provenance checks. 
        :return: AnalysisResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_provenance_python_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.post_provenance_python_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def post_provenance_python_with_http_info(self, body, **kwargs):  # noqa: E501
        """Check provenance of packages stated in an application stack.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_provenance_python_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PythonStack body: Pipfile and Pipfile.lock as used by pipenv. (required)
        :param str origin: A repository where the application stack is used. This is used for tracking as well as for automated reporting when results are available. 
        :param bool debug: Run the provenance checker in a verbose mode so developers can debug it. 
        :param bool force: Do not use cached results, always run provenance checks. 
        :return: AnalysisResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'origin', 'debug', 'force']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_provenance_python" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `post_provenance_python`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'origin' in params:
            query_params.append(('origin', params['origin']))  # noqa: E501
        if 'debug' in params:
            query_params.append(('debug', params['debug']))  # noqa: E501
        if 'force' in params:
            query_params.append(('force', params['force']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/provenance/python', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



#!/usr/bin/env python3
# thamos
# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Implementation discovery methods to detect the current environment and its configuration."""

import os
import sys
import logging
import typing
import sysconfig

import distro
import click
from thoth.analyzer import run_command

_LOGGER = logging.getLogger(__name__)
_PROC_CPU_INFO = "/proc/cpuinfo"


def discover_cuda_version(interactive: bool = False) -> typing.Optional[str]:
    """Check for CUDA version, if no CUDA is installed, return None."""
    if bool(int(os.getenv("THAMOS_DISABLE_CUDA", 0))):
        _LOGGER.debug(
            "Disabling CUDA based on THAMOS_DISABLE_CUDA environment variable that is set to %r",
            os.environ["THAMOS_DISABLE_CUDA"],
        )
        return None

    result = run_command("nvcc --version", raise_on_error=False)
    if result.return_code != 0:
        _LOGGER.info("No CUDA version detected")
        _LOGGER.debug(
            "Unable to detect CUDA version - nvcc returned non-zero version: %s",
            result.to_dict(),
        )
        return None

    lines = result.stdout.splitlines()
    version_info = lines[-1].split(",")
    if len(version_info) != 3:
        _LOGGER.debug(
            "Unable to detect CUDA version from nvcc output: %r", result.stdout
        )
        return None

    cuda_version = version_info[1].strip()[len("release "):]

    if interactive:
        cuda_version = click.prompt("Please select CUDA version", default=cuda_version)

    _LOGGER.info("Detected CUDA version: %r", cuda_version)
    return cuda_version


def discover_distribution() -> tuple:
    """Get distribution identifier and distribution version."""
    distribution, version, *_ = distro.linux_distribution(full_distribution_name=False)
    return distribution, version


def discover_python_version() -> str:
    """Discover Python version in which we run in."""
    return f"{sys.version_info.major}.{sys.version_info.minor}"


def discover_cpu() -> dict:
    """Discover CPU model, model name and family."""
    result = {
        "cpu_family": None,
        "cpu_model": None,
        "cpu_model_name": None
    }

    try:
        with open(_PROC_CPU_INFO, "r") as cpu_info_file:
            content = cpu_info_file.read().splitlines()

        for line in content:
            if line.startswith("model name") and result["cpu_model_name"] is None:
                try:
                    result["cpu_model_name"] = line.split(":")[1].strip()
                except Exception as exc:
                    _LOGGER.warning("Failed to obtain CPU model name from %s: %s", str(exc), _PROC_CPU_INFO)
            elif line.startswith("model\t") and result["cpu_model"] is None:
                try:
                    result["cpu_model"] = int(line.split(":")[1])
                except Exception as exc:
                    _LOGGER.warning("Failed to obtain CPU model from %s: %s", str(exc), _PROC_CPU_INFO)
            elif line.startswith("cpu family") and result["cpu_family"] is None:
                try:
                    result["cpu_family"] = int(line.split(":")[1])
                except Exception as exc:
                    _LOGGER.warning("Failed to obtain CPU family from %s: %s", str(exc), _PROC_CPU_INFO)

    except Exception as exc:
        _LOGGER.exception("Failed to obtain CPU specific information: %s", str(exc))

    if result["cpu_model_name"] is None:
        # Assign a text representation - unknown for config file.
        result["cpu_model_name"] = "Unknown"

    _LOGGER.info("Detected CPU: %s", ", ".join((f"{k}: {v}" for k, v in result.items())))
    return result


def discover_platform() -> str:
    """Discover platform used."""
    return sysconfig.get_platform()



# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class InlineResponse2001RuntimeEnvironments(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'os_name': 'str',
        'os_version': 'str',
        'python_version': 'str'
    }

    attribute_map = {
        'os_name': 'os_name',
        'os_version': 'os_version',
        'python_version': 'python_version'
    }

    def __init__(self, os_name=None, os_version=None, python_version=None):  # noqa: E501
        """InlineResponse2001RuntimeEnvironments - a model defined in Swagger"""  # noqa: E501
        self._os_name = None
        self._os_version = None
        self._python_version = None
        self.discriminator = None
        self.os_name = os_name
        self.os_version = os_version
        self.python_version = python_version

    @property
    def os_name(self):
        """Gets the os_name of this InlineResponse2001RuntimeEnvironments.  # noqa: E501

        Name of the operating system which is supported.  # noqa: E501

        :return: The os_name of this InlineResponse2001RuntimeEnvironments.  # noqa: E501
        :rtype: str
        """
        return self._os_name

    @os_name.setter
    def os_name(self, os_name):
        """Sets the os_name of this InlineResponse2001RuntimeEnvironments.

        Name of the operating system which is supported.  # noqa: E501

        :param os_name: The os_name of this InlineResponse2001RuntimeEnvironments.  # noqa: E501
        :type: str
        """
        if os_name is None:
            raise ValueError("Invalid value for `os_name`, must not be `None`")  # noqa: E501

        self._os_name = os_name

    @property
    def os_version(self):
        """Gets the os_version of this InlineResponse2001RuntimeEnvironments.  # noqa: E501

        Version of the operating system which is supported.  # noqa: E501

        :return: The os_version of this InlineResponse2001RuntimeEnvironments.  # noqa: E501
        :rtype: str
        """
        return self._os_version

    @os_version.setter
    def os_version(self, os_version):
        """Sets the os_version of this InlineResponse2001RuntimeEnvironments.

        Version of the operating system which is supported.  # noqa: E501

        :param os_version: The os_version of this InlineResponse2001RuntimeEnvironments.  # noqa: E501
        :type: str
        """
        if os_version is None:
            raise ValueError("Invalid value for `os_version`, must not be `None`")  # noqa: E501

        self._os_version = os_version

    @property
    def python_version(self):
        """Gets the python_version of this InlineResponse2001RuntimeEnvironments.  # noqa: E501

        Supported Python version.  # noqa: E501

        :return: The python_version of this InlineResponse2001RuntimeEnvironments.  # noqa: E501
        :rtype: str
        """
        return self._python_version

    @python_version.setter
    def python_version(self, python_version):
        """Sets the python_version of this InlineResponse2001RuntimeEnvironments.

        Supported Python version.  # noqa: E501

        :param python_version: The python_version of this InlineResponse2001RuntimeEnvironments.  # noqa: E501
        :type: str
        """
        if python_version is None:
            raise ValueError("Invalid value for `python_version`, must not be `None`")  # noqa: E501

        self._python_version = python_version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(InlineResponse2001RuntimeEnvironments, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InlineResponse2001RuntimeEnvironments):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#!/usr/bin/env python3
# thamos
# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""CLI and library configuration."""

import logging
import os
import typing

import click
import requests
import yaml

from .utils import workdir
from .discover import discover_cpu
from .discover import discover_cuda_version
from .discover import discover_distribution
from .discover import discover_python_version
from .discover import discover_platform
from .exceptions import NoApiSupported
from .exceptions import InternalError
from .exceptions import NoRuntimeEnvironmentError
from .exceptions import ConfigurationError
from .exceptions import NoProjectDirError
from .exceptions import ServiceUnavailable
from .exceptions import NoRequirementsFormatError
from urllib.parse import urljoin

_LOGGER = logging.getLogger(__name__)
_THAMOS_DISABLE_TLS_WARNING = bool(int(os.getenv("THAMOS_DISABLE_TLS_WARNING", 0)))


class _Configuration:
    """Handling of Thoth's configuration."""

    DATA_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data")
    DEFAULT_THOTH_CONFIG = os.path.join(DATA_DIR, "defaultThoth.yaml")
    CONFIG_NAME = ".thoth.yaml"
    REQUIREMENTS_FORMATS = frozenset(("pip", "pip-tools", "pip-compile", "pipenv"))
    _DEFAULT_REQUIREMENTS_FORMAT = "pipenv"

    def __init__(self):
        """Construct configuration instance."""
        self._configuration = None
        self._index_aicoe = None
        # Thoth instance to be used when explicitly said by user - the one stated in
        # configuration file will be omitted.
        self.explicit_host = None
        self.tls_verify = None
        self._api_url = None

    @property
    def api_url(self):
        """Get URL to Thoth's API."""
        if not self._api_url:
            self._api_url = self.api_discovery(self.content["host"])

        return self._api_url

    @property
    def content(self):
        """Access configuration."""
        if not self._configuration:
            self.load_config()

        return self._configuration

    @property
    def requirements_format(self) -> str:
        """Check requirements_format in configuration."""
        requirements_format = self.content.get("requirements_format")
        if not requirements_format:
            raise NoRequirementsFormatError(
                "No requirements format configuration stated in the configuration file "
                "under 'requirements_format' configuration entry"
            )

        if not isinstance(requirements_format, str):
            raise ConfigurationError("The data type for requirements_format should be str")

        if requirements_format not in ("pip", "pip-tools", "pipenv"):
            raise ValueError(f"Unknown configuration option for requirements format: {requirements_format!r}")

        return requirements_format

    def get_thoth_version(self) -> str:
        """Get version of Thoth backend."""
        _LOGGER.debug("Contacting Thoth at %r to receive version information", self.api_url)
        response = requests.head(self.api_url, verify=self.tls_verify)
        response.raise_for_status()
        return response.headers.get("X-Thoth-Version", "Not Available")

    def config_file_exists(self) -> bool:
        """Check if configuration file exists."""
        try:
            with workdir(self.CONFIG_NAME):
                return True
        except NoProjectDirError:
            return False

    def load_config(self):
        """Load configuration file."""
        with workdir(config.CONFIG_NAME):
            with open(config.CONFIG_NAME, "r") as config_file:
                self._configuration = config_file.read()

                if int(os.getenv("THAMOS_CONFIG_EXPAND_ENV", 0)):
                    _LOGGER.info("Expanding configuration file based on environment variables")
                    self._configuration = self._configuration.format(**os.environ)

                self._configuration = yaml.safe_load(self._configuration)

    def create_default_config(self, template: str = None, nowrite: bool = False) -> typing.Optional[dict]:
        """Place default configuration into the current directory."""
        if not os.path.isdir(".git"):
            _LOGGER.warning("Configuration file is not created in the root of git repo")

        template = template or self.DEFAULT_THOTH_CONFIG
        _LOGGER.debug(
            "Reading configuration from %r", template
        )
        with open(template, "r") as default_config_file:
            default_config = default_config_file.read()

        _LOGGER.info("Discovering host runtime environment")

        cpu_info = discover_cpu()
        cuda_version = discover_cuda_version()
        # Add quotes for textual representation in the config file.
        cuda_version = f"'{cuda_version}" if cuda_version is not None else 'null'
        os_name, os_version = discover_distribution()
        python_version = discover_python_version()
        platform = discover_platform()

        requirements_format = os.getenv("THAMOS_REQUIREMENTS_FORMAT", self._DEFAULT_REQUIREMENTS_FORMAT)
        if requirements_format not in self.REQUIREMENTS_FORMATS:
            # This avoids possibly dangerous environment variable expansion.
            _LOGGER.warning(
                "Unknown requirements format specified, forcing %r: %r",
                self._DEFAULT_REQUIREMENTS_FORMAT,
                requirements_format
            )
            requirements_format = self._DEFAULT_REQUIREMENTS_FORMAT

        expand_env = bool(int(os.getenv("THAMOS_CONFIG_EXPAND_ENV", 0)))
        default_config = default_config.format(
            cuda_version=cuda_version,
            os_name=os_name,
            os_version=os_version,
            python_version=python_version,
            platform=platform,
            requirements_format=requirements_format,
            **cpu_info,
            **(os.environ if expand_env else {}),
        )

        if not nowrite:
            _LOGGER.debug(
                "Writing configuration file to %r",
                os.path.join(os.getcwd(), self.CONFIG_NAME),
            )

            with open(self.CONFIG_NAME, "w") as config_file:
                config_file.write(default_config)
        else:
            return yaml.safe_load(default_config)

    @staticmethod
    def open_config_file():
        """Open Thoth's configuration file."""
        with workdir(config.CONFIG_NAME):
            _LOGGER.debug("Opening configuration file %r", config.CONFIG_NAME)
            click.edit(filename=config.CONFIG_NAME)

    def list_runtime_environments(self):
        """List available runtime environments."""
        return self.content.get("runtime_environments", [])

    def get_runtime_environment(self, name: str = None) -> typing.Optional[dict]:
        """Get runtime environment, retrieve the first runtime environment (the default one) if no name is provided."""
        content = self.content
        if "runtime_environments" not in content:
            raise NoRuntimeEnvironmentError(
                "No runtime environment configuration stated in the configuration file "
                "under 'runtime_environments' configuration entry"
            )

        if not isinstance(content["runtime_environments"], list):
            raise ConfigurationError("The data type for requirements_format should be list")

        to_return = None
        seen_names = set()
        for idx, runtime_environment in enumerate(content["runtime_environments"]):
            if not isinstance(runtime_environment, dict):
                raise ConfigurationError(
                    "Unknown runtime configuration entry, runtime configuration should be "
                    "a dictionary; got: %r",
                    runtime_environment,
                )

            # We explicitly iterate over all entries to perform the following sanity checks.
            current_name = runtime_environment.get("name")
            if current_name is not None and current_name in seen_names:
                raise ConfigurationError(
                    "Multiple configuration options with name %r found in the configuration file",
                    current_name,
                )

            if idx > 0 and current_name is None:
                raise ConfigurationError(
                    "Assign explicitly name for each configuration entry if there are multiple "
                    "runtime configuration options to distinguish between them"
                )

            if current_name is not None:
                seen_names.add(current_name)

            if name is None and idx == 0:
                # Return the first one by default.
                to_return = runtime_environment
            elif current_name == name:
                # Return by name.
                to_return = runtime_environment

        if to_return is None and len(content["runtime_environments"]) > 0:
            if name is not None:
                raise NoRuntimeEnvironmentError(
                    f"No runtime environment with name {name!r} was found in the configuration file; "
                    f"configured runtime environment names: {','.join(seen_names)}"
                )

            raise NoRuntimeEnvironmentError(
                "No runtime environment configuration was found"
            )

        return to_return

    def api_discovery(self, host: str = None) -> str:
        """Discover API versions available, return the most recent one supported by client and server."""
        api_url = urljoin("https://" + host, "/api/v1")
        self.tls_verify = (
            self.tls_verify
            if self.tls_verify is not None
            else self.content.get("tls_verify", True)
        )

        if not self.tls_verify and not _THAMOS_DISABLE_TLS_WARNING:
            _LOGGER.warning(
                "TLS verification turned off, its highly recommended to use a secured connection, "
                "see configuration file for configuration options"
            )

        response = requests.get(
            api_url, verify=self.tls_verify, headers={"Accept": "application/json"}
        )

        try:
            response.raise_for_status()
        except Exception as exc:
            if response.status_code == 503:
                _LOGGER.error("Thoth service at %r is unavailable (HTTP 503)", api_url)

            raise NoApiSupported(
                "Server does not support API v1 required by Thamos client"
            ) from exc

        self._api_url = api_url
        return self._api_url


config = _Configuration()



# coding: utf-8

# flake8: noqa
"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import models into model package
from thamos.swagger_client.models.advise_input import AdviseInput
from thamos.swagger_client.models.advise_input_library_usage import AdviseInputLibraryUsage
from thamos.swagger_client.models.analysis_listing_response import AnalysisListingResponse
from thamos.swagger_client.models.analysis_log_response import AnalysisLogResponse
from thamos.swagger_client.models.analysis_response import AnalysisResponse
from thamos.swagger_client.models.analysis_response_error import AnalysisResponseError
from thamos.swagger_client.models.analysis_result_response import AnalysisResultResponse
from thamos.swagger_client.models.analysis_result_response_metadata import AnalysisResultResponseMetadata
from thamos.swagger_client.models.analysis_result_response_metadata_distribution import AnalysisResultResponseMetadataDistribution
from thamos.swagger_client.models.analysis_result_response_metadata_distribution_version_parts import AnalysisResultResponseMetadataDistributionVersionParts
from thamos.swagger_client.models.analysis_result_response_metadata_python import AnalysisResultResponseMetadataPython
from thamos.swagger_client.models.analysis_status_response import AnalysisStatusResponse
from thamos.swagger_client.models.analysis_status_response_status import AnalysisStatusResponseStatus
from thamos.swagger_client.models.analysis_unfinished_result_response import AnalysisUnfinishedResultResponse
from thamos.swagger_client.models.build import Build
from thamos.swagger_client.models.build_analysis_response import BuildAnalysisResponse
from thamos.swagger_client.models.build_analysis_response_base_image_analysis import BuildAnalysisResponseBaseImageAnalysis
from thamos.swagger_client.models.build_analysis_response_error import BuildAnalysisResponseError
from thamos.swagger_client.models.build_analysis_response_error_base_image_analysis import BuildAnalysisResponseErrorBaseImageAnalysis
from thamos.swagger_client.models.build_build_log import BuildBuildLog
from thamos.swagger_client.models.image_metadata_response import ImageMetadataResponse
from thamos.swagger_client.models.inline_response200 import InlineResponse200
from thamos.swagger_client.models.inline_response2001 import InlineResponse2001
from thamos.swagger_client.models.inline_response2001_runtime_environments import InlineResponse2001RuntimeEnvironments
from thamos.swagger_client.models.inline_response2002 import InlineResponse2002
from thamos.swagger_client.models.inline_response2002_software_environments import InlineResponse2002SoftwareEnvironments
from thamos.swagger_client.models.inline_response200_hardware_environments import InlineResponse200HardwareEnvironments
from thamos.swagger_client.models.inline_response200_parameters import InlineResponse200Parameters
from thamos.swagger_client.models.kebechet_webhook_input import KebechetWebhookInput
from thamos.swagger_client.models.log import Log
from thamos.swagger_client.models.python_package_dependencies import PythonPackageDependencies
from thamos.swagger_client.models.python_package_dependencies_error import PythonPackageDependenciesError
from thamos.swagger_client.models.python_package_dependencies_inner import PythonPackageDependenciesInner
from thamos.swagger_client.models.python_package_indexes import PythonPackageIndexes
from thamos.swagger_client.models.python_package_indexes_inner import PythonPackageIndexesInner
from thamos.swagger_client.models.python_package_metadata_response import PythonPackageMetadataResponse
from thamos.swagger_client.models.python_package_metadata_response_error import PythonPackageMetadataResponseError
from thamos.swagger_client.models.python_packages_count_info_response import PythonPackagesCountInfoResponse
from thamos.swagger_client.models.python_packages_count_info_response_error import PythonPackagesCountInfoResponseError
from thamos.swagger_client.models.python_stack import PythonStack
from thamos.swagger_client.models.qeb_hwt_thamos_advise_input import QebHwtThamosAdviseInput
from thamos.swagger_client.models.runtime_environment import RuntimeEnvironment



# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class AnalysisResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'analysis_id': 'str',
        'cached': 'bool',
        'parameters': 'object'
    }

    attribute_map = {
        'analysis_id': 'analysis_id',
        'cached': 'cached',
        'parameters': 'parameters'
    }

    def __init__(self, analysis_id=None, cached=None, parameters=None):  # noqa: E501
        """AnalysisResponse - a model defined in Swagger"""  # noqa: E501
        self._analysis_id = None
        self._cached = None
        self._parameters = None
        self.discriminator = None
        self.analysis_id = analysis_id
        self.cached = cached
        self.parameters = parameters

    @property
    def analysis_id(self):
        """Gets the analysis_id of this AnalysisResponse.  # noqa: E501

        An id of submitted analysis for checking its status and its results.   # noqa: E501

        :return: The analysis_id of this AnalysisResponse.  # noqa: E501
        :rtype: str
        """
        return self._analysis_id

    @analysis_id.setter
    def analysis_id(self, analysis_id):
        """Sets the analysis_id of this AnalysisResponse.

        An id of submitted analysis for checking its status and its results.   # noqa: E501

        :param analysis_id: The analysis_id of this AnalysisResponse.  # noqa: E501
        :type: str
        """
        if analysis_id is None:
            raise ValueError("Invalid value for `analysis_id`, must not be `None`")  # noqa: E501

        self._analysis_id = analysis_id

    @property
    def cached(self):
        """Gets the cached of this AnalysisResponse.  # noqa: E501

        If set to true the given analysis was picked from cache.   # noqa: E501

        :return: The cached of this AnalysisResponse.  # noqa: E501
        :rtype: bool
        """
        return self._cached

    @cached.setter
    def cached(self, cached):
        """Sets the cached of this AnalysisResponse.

        If set to true the given analysis was picked from cache.   # noqa: E501

        :param cached: The cached of this AnalysisResponse.  # noqa: E501
        :type: bool
        """
        if cached is None:
            raise ValueError("Invalid value for `cached`, must not be `None`")  # noqa: E501

        self._cached = cached

    @property
    def parameters(self):
        """Gets the parameters of this AnalysisResponse.  # noqa: E501

        Parameters echoed back to user (with default parameters if omitted).   # noqa: E501

        :return: The parameters of this AnalysisResponse.  # noqa: E501
        :rtype: object
        """
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        """Sets the parameters of this AnalysisResponse.

        Parameters echoed back to user (with default parameters if omitted).   # noqa: E501

        :param parameters: The parameters of this AnalysisResponse.  # noqa: E501
        :type: object
        """
        if parameters is None:
            raise ValueError("Invalid value for `parameters`, must not be `None`")  # noqa: E501

        self._parameters = parameters

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AnalysisResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AnalysisResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class AnalysisUnfinishedResultResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'error': 'str',
        'parameters': 'object',
        'status': 'AnalysisStatusResponse'
    }

    attribute_map = {
        'error': 'error',
        'parameters': 'parameters',
        'status': 'status'
    }

    def __init__(self, error=None, parameters=None, status=None):  # noqa: E501
        """AnalysisUnfinishedResultResponse - a model defined in Swagger"""  # noqa: E501
        self._error = None
        self._parameters = None
        self._status = None
        self.discriminator = None
        self.error = error
        self.parameters = parameters
        self.status = status

    @property
    def error(self):
        """Gets the error of this AnalysisUnfinishedResultResponse.  # noqa: E501

        Error information for user.  # noqa: E501

        :return: The error of this AnalysisUnfinishedResultResponse.  # noqa: E501
        :rtype: str
        """
        return self._error

    @error.setter
    def error(self, error):
        """Sets the error of this AnalysisUnfinishedResultResponse.

        Error information for user.  # noqa: E501

        :param error: The error of this AnalysisUnfinishedResultResponse.  # noqa: E501
        :type: str
        """
        if error is None:
            raise ValueError("Invalid value for `error`, must not be `None`")  # noqa: E501

        self._error = error

    @property
    def parameters(self):
        """Gets the parameters of this AnalysisUnfinishedResultResponse.  # noqa: E501

        Parameters echoed back to user for debugging.  # noqa: E501

        :return: The parameters of this AnalysisUnfinishedResultResponse.  # noqa: E501
        :rtype: object
        """
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        """Sets the parameters of this AnalysisUnfinishedResultResponse.

        Parameters echoed back to user for debugging.  # noqa: E501

        :param parameters: The parameters of this AnalysisUnfinishedResultResponse.  # noqa: E501
        :type: object
        """
        if parameters is None:
            raise ValueError("Invalid value for `parameters`, must not be `None`")  # noqa: E501

        self._parameters = parameters

    @property
    def status(self):
        """Gets the status of this AnalysisUnfinishedResultResponse.  # noqa: E501


        :return: The status of this AnalysisUnfinishedResultResponse.  # noqa: E501
        :rtype: AnalysisStatusResponse
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this AnalysisUnfinishedResultResponse.


        :param status: The status of this AnalysisUnfinishedResultResponse.  # noqa: E501
        :type: AnalysisStatusResponse
        """
        if status is None:
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AnalysisUnfinishedResultResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AnalysisUnfinishedResultResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#!/usr/bin/env python3
# thamos
# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Command line interface Thamos for interaction with Thoth."""

import logging
import typing
import os
import sys
from shutil import get_terminal_size
import json
from functools import wraps
from typing import Tuple
from typing import Optional

import yaml
from texttable import Texttable
import click
from termcolor import colored
import daiquiri
from thoth.python import Project
from thoth.common import ThothAdviserIntegrationEnum
from thamos.exceptions import NoProjectDirError
from thamos.config import config as configuration
from thamos.lib import advise as thoth_advise
from thamos.lib import provenance_check as thoth_provenance_check
from thamos.lib import get_log
from thamos.lib import get_status
from thamos.utils import workdir
from thamos import __version__ as thamos_version

# Suppress anoying errors when name not known (disable_warnings() does not work here).
logging.getLogger("urllib3.connectionpool").setLevel(logging.ERROR)
daiquiri.setup(level=logging.INFO)
_LOGGER = logging.getLogger("thamos")

_EMOJI = {
    "WARNING": colored("\u26a0\ufe0f WARNING", "yellow"),
    "ERROR": colored("\u274c ERROR", "red", attrs=["bold"]),
    "INFO": colored("\u2714\ufe0f INFO", "green"),
    "LATEST": colored("\U0001f44c LATEST", "green"),
    "CVE": colored("\u2620\uFE0F  CVE \u2620\uFE0F", "red"),
}

# Align of columns in table - default is left, values stated here are adjusted otherwise.
_TABLE_COLS_ALIGN = {
    "type": "c",
    "severity": "c",
    "package_name": "c",
    "package_version": "r",
}


def handle_cli_exception(func: typing.Callable) -> typing.Callable:
    """Suppress exception in CLI if debug mode was not turned on."""
    @wraps(func)
    def wrapper(ctx, *args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as exc:
            if ctx.parent.params["verbose"]:
                raise

            _LOGGER.error(str(exc))
            sys.exit(1)

    return wrapper


def _load_files(requirements_format: str) -> Tuple[str, Optional[str]]:
    """Load Pipfile/Pipfile.lock or requirements.in/txt from the current directory."""
    if requirements_format == "pipenv":
        project = Project.from_files(without_pipfile_lock=not os.path.exists("Pipfile.lock"))
    elif requirements_format in ("pip", "pip-tools", "pip-compile"):
        project = Project.from_pip_compile_files(allow_without_lock=True)
    else:
        raise ValueError(f"Unknown configuration option for requirements format: {requirements_format!r}")
    return project.pipfile.to_string(), project.pipfile_lock.to_string() if project.pipfile_lock else None


def _write_files(requirements: str, requirements_lock: str, requirements_format: str) -> None:
    """Write content of Pipfile/Pipfile.lock or requirements.in/txt to the current directory."""
    project = Project.from_dict(requirements, requirements_lock)
    if requirements_format == "pipenv":
        _LOGGER.debug("Writing to Pipfile/Pipfile.lock in %r", os.getcwd())
        project.to_files()
    elif requirements_format in ("pip", "pip-tools", "pip-compile"):
        _LOGGER.debug("Writing to requirements.in/requirements.txt in %r", os.getcwd())
        project.to_pip_compile_files()
        _LOGGER.debug("No changes to Pipfile to write")
    else:
        raise ValueError(
            f"Unknown requirements format, supported are 'pipenv' and 'pip': {requirements_format!r}"
        )


def _print_header(header: str) -> None:
    """Print header to terminal respecting terminal size."""
    terminal_size = get_terminal_size()
    padding = (terminal_size.columns - len(header) - 2) // 2
    click.echo(padding * " " + header)
    click.echo(padding * " " + "=" * len(header) + "  \n")


def _write_configuration(
    advised_configuration: dict,
    recommendation_type: str = None,
    limit_latest_versions: int = None,
    dev: bool = False,
) -> None:
    """Create thoth configuration file."""
    if not advised_configuration:
        _LOGGER.debug("No advises on configuration, nothing to adjust")
        return

    if "name" not in advised_configuration:
        _LOGGER.error(
            "Cannot adjust Thoth's configuration based on advises: No name found in Thoth's configuration entry"
        )
        return

    _LOGGER.debug("Reading Thoth's configuration file")
    with open(".thoth.yaml", "r") as thoth_yaml_file:
        content = yaml.safe_load(thoth_yaml_file.read())

    for idx, runtime_environment_entry in enumerate(
        content.get("runtime_environments", [])
    ):
        if runtime_environment_entry.get("name") == advised_configuration["name"]:
            _LOGGER.debug(
                "Adjusting configuration entry for %r based on recommendations",
                advised_configuration["name"],
            )
            runtime_environment_entry = advised_configuration
            if recommendation_type:
                runtime_environment_entry["recommendation_type"] = recommendation_type
            if limit_latest_versions:
                runtime_environment_entry[
                    "limit_latest_versions"
                ] = limit_latest_versions
            runtime_environment_entry["dev"] = dev
            content["runtime_environments"][idx] = runtime_environment_entry
            break
    else:
        _LOGGER.error(
            "Cannot adjust Thoth's configuration based on advises: No runtime environment entry with name %r found",
            advised_configuration["name"],
        )
        return

    _LOGGER.debug("Writing adjusted Thoth's configuration file")
    with open(".thoth.yaml", "w") as thoth_yaml_file:
        yaml.safe_dump(content, thoth_yaml_file)


def _print_report(report: dict, json_output: bool = False):
    """Print reasoning to user."""
    if json_output:
        click.echo(json.dumps(report, sort_keys=True, indent=2))
        return

    table = Texttable(max_width=get_terminal_size().columns)
    table.set_deco(Texttable.HEADER | Texttable.VLINES)

    header = set()
    to_remove = set()
    for item in report:
        header = header.union(set(item.keys()))
        to_remove = to_remove.union(
            set(i for i, v in item.items() if isinstance(v, dict))
        )

    # Remove fields that can be an array - these are addition details that are supressed from the table output.
    header = header - to_remove

    header = list(sorted(header))
    table.set_cols_align([_TABLE_COLS_ALIGN.get(column, "l") for column in header])
    table.header([item[0].upper() + item[1:].replace("_", " ") for item in header])

    for item in report:
        row = []
        for column in header:
            entry = item.get(column, "-")

            if not bool(int(os.getenv("THAMOS_NO_EMOJI", 0))) and isinstance(
                entry, str
            ):
                entry = _EMOJI.get(entry, entry)

            if isinstance(entry, list):
                entry = ", ".join(entry)

            row.append(entry)

        table.add_row(row)

    click.echo(table.draw())


@click.group()
@click.pass_context
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    envvar="THAMOS_VERBOSE",
    help="Be verbose about what's going on.",
)
@click.option(
    "--workdir",
    "-d",
    type=str,
    default=None,
    help="Adjust working directory for sub-commands.",
)
@click.option(
    "--thoth-host",
    "-t",
    type=str,
    default=None,
    help="Use selected host instead of the one stated in the configuration file.",
)
def cli(ctx=None, verbose: bool = False, workdir: str = None, thoth_host: str = None):
    """CLI tool for interacting with Thoth."""
    if ctx:
        ctx.auto_envvar_prefix = "THAMOS"

    if verbose:
        _LOGGER.setLevel(logging.DEBUG)
        _LOGGER.debug("Debug mode turned on")
        _LOGGER.debug("Thamos version: %r", thamos_version)

    if workdir:
        _LOGGER.debug("Changing working directory into %r", workdir)
        os.chdir(workdir)

    if thoth_host:
        configuration.explicit_host = thoth_host

    # Turn on progressbar explicitly here if it was not turned off by user. If Thamos is used
    # as a library, progressbar is not shown as one would expect.
    os.environ["THAMOS_NO_PROGRESSBAR"] = os.getenv("THAMOS_NO_PROGRESSBAR", "0")


@cli.command("version")
@click.pass_context
@click.option(
    "--json", "-j", "json_output", is_flag=True, help="Print output in JSON format."
)
def _print_version(ctx, json_output: bool = False):
    """Print Thamos and Thoth version and exit."""
    api_url = None
    thoth_version = None
    try:
        thoth_version = configuration.get_thoth_version()
        api_url = configuration.api_url
    except NoProjectDirError as exc:
        _LOGGER.warning("Cannot obtain Thoth backend information: %s", str(exc))

    if json_output:
        click.echo(
            json.dumps(
                {
                    "thamos_version": thamos_version,
                    "thoth_version": thoth_version,
                    "thoth_api_url": configuration.api_url,
                },
                indent=2,
            )
        )
    else:
        click.echo(f"Thamos Client version: {thamos_version!s}")
        click.echo(
            f"Thoth API {api_url if api_url is not None else 'N/A'}: {thoth_version if thoth_version else 'N/A'}"
        )

    ctx.exit(0)


@cli.command("advise")
@click.option(
    "--debug",
    is_flag=True,
    envvar="THAMOS_DEBUG",
    help="Run analysis in debug mode on Thoth.",
)
@click.option(
    "--no-write",
    "-W",
    is_flag=True,
    help="Do not write results to files, just print them.",
)
@click.option(
    "--recommendation-type",
    "-t",
    type=str,
    metavar="RECOMMENDATION_TYPE",
    envvar="THAMOS_RECOMMENDATION_TYPE",
    help="Use selected recommendation type, do not load it from Thoth's config file.",
)
@click.option(
    "--no-wait",
    is_flag=True,
    help="Do not wait for analysis to finish, just submit it.",
)
@click.option(
    "--no-static-analysis",
    is_flag=True,
    envvar="THAMOS_NO_STATIC_ANALYSIS",
    help="Do not perform static analysis of source code files.",
)
@click.option(
    "--json", "-j", "json_output", is_flag=True, help="Print output in JSON format."
)
@click.option(
    "--force",
    is_flag=True,
    envvar="THAMOS_FORCE",
    help="Force analysis run bypassing server-side cache.",
)
@click.option(
    "--runtime-environment",
    "-r",
    type=str,
    default=None,
    metavar="NAME",
    envvar="THAMOS_RUNTIME_ENVIRONMENT",
    help="Specify explicitly runtime environment to get recommendations for; "
    "defaults to the first entry in the configuration file.",
)
@click.option(
    "--limit-latest-versions",
    type=int,
    default=None,
    metavar="COUNT",
    envvar="THAMOS_LIMIT_LATEST_VERSIONS",
    help="Specify number of latest versions for each package to consider.",
)
@click.option(
    "--dev/--no-dev",
    envvar="THAMOS_DEV",
    is_flag=True,
    default=False,
    show_default=True,
    help="Consider or do not consider development dependencies during the resolution.",
)
def advise(
    debug: bool = False,
    no_write: bool = False,
    recommendation_type: str = None,
    runtime_environment: str = None,
    no_wait: bool = False,
    no_static_analysis: bool = False,
    json_output: bool = False,
    limit_latest_versions: int = None,
    force: bool = False,
    dev: bool = False,
):
    """Ask Thoth for recommendations on application stack."""
    with workdir():
        pipfile, pipfile_lock = _load_files(requirements_format=configuration.requirements_format)

        # In CLI we always call to obtain only the best software stack (count is implicitly set to 1).
        results = thoth_advise(
            pipfile,
            pipfile_lock,
            recommendation_type=recommendation_type,
            runtime_environment_name=runtime_environment,
            debug=debug,
            nowait=no_wait,
            force=force,
            source_type=ThothAdviserIntegrationEnum.CLI,
            limit_latest_versions=limit_latest_versions,
            no_static_analysis=no_static_analysis,
            dev=dev,
        )

        if not results:
            return sys.exit(2)

        if no_wait:
            # Echo the analysis id to user when not waiting.
            click.echo(results)
            sys.exit(0)

        result, error = results
        if error:
            if json_output:
                print({"error": result["error_msg"]})
            else:
                print(result["error_msg"])
            sys.exit(4)

        if not no_write:
            # Print report of the best one - thus index zero.
            if result["report"] and result["report"]["products"]:
                if result["report"]["products"][0]["justification"]:
                    _print_header("Recommended stack report")
                    _print_report(result["report"]["products"][0]["justification"], json_output=json_output)
                else:
                    click.echo("No justification was made for the recommended stack")

            if result["report"] and result["report"]["stack_info"]:
                _print_header("Application stack guidance")
                _print_report(result["report"]["stack_info"], json_output=json_output)

            pipfile = result["report"]["products"][0]["project"]["requirements"]
            pipfile_lock = result["report"]["products"][0]["project"]["requirements_locked"]
            _write_configuration(
                result["report"]["products"][0]["advised_runtime_environment"],
                recommendation_type,
                limit_latest_versions,
                dev,
            )
            _write_files(pipfile, pipfile_lock, configuration.requirements_format)
        else:
            click.echo(json.dumps(result, indent=2))

    sys.exit(0)


@cli.command("provenance-check")
@click.option(
    "--debug",
    is_flag=True,
    envvar="THAMOS_DEBUG",
    help="Run analysis in debug mode on Thoth.",
)
@click.option(
    "--json", "-j", "json_output", is_flag=True, help="Print output in JSON format."
)
@click.option(
    "--no-wait",
    is_flag=True,
    help="Do not wait for analysis to finish, just submit it.",
)
@click.option(
    "--force",
    is_flag=True,
    envvar="THAMOS_FORCE",
    help="Force analysis run bypassing server-side cache.",
)
@click.pass_context
@handle_cli_exception
def provenance_check(
    debug: bool = False,
    no_wait: bool = False,
    json_output: bool = False,
    force: bool = False,
):
    """Check provenance of installed packages."""
    with workdir():
        if configuration.requirements_format != "pipenv":
            raise ValueError("Provenance checks are available only for requirements managed by Pipenv")

        pipfile, pipfile_lock = _load_files("pipenv")
        if not pipfile_lock:
            _LOGGER.error("No Pipfile.lock found - provenance cannot be checked")
            sys.exit(3)

        results = thoth_provenance_check(
            pipfile, pipfile_lock, debug=debug, nowait=no_wait, force=force
        )
        if not results:
            sys.exit(2)

        if no_wait:
            # Echo the analysis id to user when nowait.
            click.echo(results)
            sys.exit(0)

        report, error = results
        _print_report(report, json_output=json_output) if report else _LOGGER.info(
            "Provenance check passed!"
        )

        if error:
            sys.exit(5)

        if any(item.get("type") == "ERROR" for item in report):
            sys.exit(4)

        return 0


@cli.command("log")
@click.argument("analysis_id", type=str, required=False)
def log(analysis_id: str = None):
    """Get log of running or finished analysis.

    If ANALYSIS_ID is not provided, there will be used last analysis id, if noted by Thamos.
    """
    if not analysis_id:
        with workdir():
            log_str = get_log()
    else:
        log_str = get_log(analysis_id)

    click.echo(log_str)


@cli.command("status")
@click.argument("analysis_id", type=str, required=False)
@click.option(
    "--output-format",
    "-o",
    type=click.Choice(["json", "yaml", "table"]),
    default="table",
    help="Specify output format for the status report.",
)
def status(analysis_id: str = None, output_format: str = None):
    """Get status of an analysis.

    If ANALYSIS_ID is not provided, there will be used last analysis id, if noted by Thamos.
    """
    if not analysis_id:
        with workdir():
            status_dict = get_status()
    else:
        status_dict = get_status(analysis_id)

    if not output_format or output_format == "table":
        table = Texttable(max_width=get_terminal_size().columns)
        table.set_deco(Texttable.VLINES)
        table.add_rows(list(status_dict.items()), header=False)
        output = table.draw()
    elif output_format == "json":
        output = json.dumps(status_dict, indent=2)
    elif output_format == "yaml":
        output = yaml.safe_dump(status_dict, default_flow_style=False)
    else:
        raise NotImplementedError(f"Unknown output format {output_format}")

    click.echo(output)


@cli.command("config")
@click.option(
    "--no-interactive",
    "-I",
    envvar="THAMOS_NO_INTERACTIVE",
    is_flag=True,
    help="Do not open editor with configuration.",
)
@click.option(
    "--template",
    "-t",
    metavar="FILE",
    type=str,
    envvar="THAMOS_CONFIG_TEMPLATE",
    help="Template which should be used instead of the default one.",
)
def config(no_interactive: bool = False, template: str = None):
    """Adjust Thamos and Thoth remote configuration.

    Perform autodiscovery of available hardware and software on the host and
    create a default configuration for Thoth (placed into .thoth.yaml).
    """
    if template:
        _LOGGER.info(
            "Creating configuration file from a configuration template %r", template
        )
        configuration.create_default_config(template)

        if not no_interactive:
            configuration.open_config_file()

        return

    if not configuration.config_file_exists():
        _LOGGER.info(
            "No configuration file found, creating one from a configuration template from %s",
            "a default template" if template is None else template,
        )
        configuration.create_default_config(template)

    if not no_interactive:
        configuration.open_config_file()
    elif no_interactive:
        _LOGGER.info(
            "Configuration file already present, no action performed in non-interactive mode"
        )


if __name__ == "__main__":
    cli()



import os
import sys
from setuptools import setup, find_packages
from pathlib import Path
from setuptools.command.test import test as TestCommand


def get_install_requires():
    with open('requirements.txt', 'r') as requirements_file:
        return [req for req in requirements_file.readlines() if req]


def get_version():
    with open(os.path.join('thamos', '__init__.py')) as f:
        content = f.readlines()

    for line in content:
        if line.startswith('__version__ ='):
            # dirty, remove trailing and leading chars
            return line.split(' = ')[1][1:-2]
    raise ValueError("No version identifier found")


class Test(TestCommand):
    """Introduce test command to run testsuite using pytest."""

    _IMPLICIT_PYTEST_ARGS = ['tests/', '--timeout=2', '--cov=./thamos', '--capture=no', '--verbose', '-l', '-s', '-vv']

    user_options = [
        ('pytest-args=', 'a', "Arguments to pass into py.test")
    ]

    def initialize_options(self):
        super().initialize_options()
        self.pytest_args = None

    def finalize_options(self):
        super().finalize_options()
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        passed_args = list(self._IMPLICIT_PYTEST_ARGS)

        if self.pytest_args:
            self.pytest_args = [arg for arg in self.pytest_args.split() if arg]
            passed_args.extend(self.pytest_args)

        sys.exit(pytest.main(passed_args))


VERSION = get_version()
setup(
    name='thamos',
    entry_points={
        'console_scripts': ['thamos=thamos.cli:cli']
    },
    version=VERSION,
    package_data={
        'thamos': [
            os.path.join('data', '*.yaml')
        ]
    },
    include_package_data=True,
    description='A CLI tool and library for interacting with Thoth',
    long_description=Path('README.rst').read_text(),
    author='Fridolin Pokorny',
    author_email='fridolin@redhat.com',
    license='GPLv3+',
    packages=find_packages(),
    long_description_content_type="text/x-rst",
    cmdclass={'test': Test},
    install_requires=get_install_requires(),
    command_options={
        "build_sphinx": {
            "version": ("setup.py", VERSION),
            "release": ("setup.py", VERSION),
        }
    },
)



# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from thamos.swagger_client.api_client import ApiClient


class ImageAnalysisApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_analyze(self, analysis_id, **kwargs):  # noqa: E501
        """Retrieve an analyzer result.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: Id of analysis that results should be retrieved. (required)
        :return: AnalysisResultResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_analyze_with_http_info(analysis_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_analyze_with_http_info(analysis_id, **kwargs)  # noqa: E501
            return data

    def get_analyze_with_http_info(self, analysis_id, **kwargs):  # noqa: E501
        """Retrieve an analyzer result.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze_with_http_info(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: Id of analysis that results should be retrieved. (required)
        :return: AnalysisResultResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['analysis_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_analyze" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'analysis_id' is set
        if ('analysis_id' not in params or
                params['analysis_id'] is None):
            raise ValueError("Missing the required parameter `analysis_id` when calling `get_analyze`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'analysis_id' in params:
            path_params['analysis_id'] = params['analysis_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/analyze/{analysis_id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisResultResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_analyze_by_hash(self, image_hash, **kwargs):  # noqa: E501
        """Retrieve an analyzer result.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze_by_hash(image_hash, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image_hash: Image hash for identifying image (including hash type, now supported only \"sha256\"). (required)
        :return: AnalysisResultResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_analyze_by_hash_with_http_info(image_hash, **kwargs)  # noqa: E501
        else:
            (data) = self.get_analyze_by_hash_with_http_info(image_hash, **kwargs)  # noqa: E501
            return data

    def get_analyze_by_hash_with_http_info(self, image_hash, **kwargs):  # noqa: E501
        """Retrieve an analyzer result.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze_by_hash_with_http_info(image_hash, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image_hash: Image hash for identifying image (including hash type, now supported only \"sha256\"). (required)
        :return: AnalysisResultResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['image_hash']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_analyze_by_hash" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'image_hash' is set
        if ('image_hash' not in params or
                params['image_hash'] is None):
            raise ValueError("Missing the required parameter `image_hash` when calling `get_analyze_by_hash`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'image_hash' in params:
            path_params['image_hash'] = params['image_hash']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/analyze/by-hash/{image_hash}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisResultResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_analyze_log(self, analysis_id, **kwargs):  # noqa: E501
        """Show logs of an analysis.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze_log(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisLogResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_analyze_log_with_http_info(analysis_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_analyze_log_with_http_info(analysis_id, **kwargs)  # noqa: E501
            return data

    def get_analyze_log_with_http_info(self, analysis_id, **kwargs):  # noqa: E501
        """Show logs of an analysis.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze_log_with_http_info(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisLogResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['analysis_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_analyze_log" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'analysis_id' is set
        if ('analysis_id' not in params or
                params['analysis_id'] is None):
            raise ValueError("Missing the required parameter `analysis_id` when calling `get_analyze_log`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'analysis_id' in params:
            path_params['analysis_id'] = params['analysis_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/analyze/{analysis_id}/log', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisLogResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_analyze_status(self, analysis_id, **kwargs):  # noqa: E501
        """Show analysis status.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze_status(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisStatusResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_analyze_status_with_http_info(analysis_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_analyze_status_with_http_info(analysis_id, **kwargs)  # noqa: E501
            return data

    def get_analyze_status_with_http_info(self, analysis_id, **kwargs):  # noqa: E501
        """Show analysis status.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_analyze_status_with_http_info(analysis_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str analysis_id: An id of requested analysis. (required)
        :return: AnalysisStatusResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['analysis_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_analyze_status" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'analysis_id' is set
        if ('analysis_id' not in params or
                params['analysis_id'] is None):
            raise ValueError("Missing the required parameter `analysis_id` when calling `get_analyze_status`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'analysis_id' in params:
            path_params['analysis_id'] = params['analysis_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/analyze/{analysis_id}/status', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisStatusResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_analyze(self, **kwargs):  # noqa: E501
        """Retrieve a list of document ids for analyzer results.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_analyze(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int page: Page offset in pagination.
        :return: AnalysisListingResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_analyze_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.list_analyze_with_http_info(**kwargs)  # noqa: E501
            return data

    def list_analyze_with_http_info(self, **kwargs):  # noqa: E501
        """Retrieve a list of document ids for analyzer results.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_analyze_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int page: Page offset in pagination.
        :return: AnalysisListingResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['page']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_analyze" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/analyze', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisListingResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_software_environment_analyses_for_build(self, environment_name, **kwargs):  # noqa: E501
        """List analyses for the given software environment for build.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environment_analyses_for_build(environment_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str environment_name: Software environment name for run for which analyses should be retrieved.  (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_software_environment_analyses_for_build_with_http_info(environment_name, **kwargs)  # noqa: E501
        else:
            (data) = self.list_software_environment_analyses_for_build_with_http_info(environment_name, **kwargs)  # noqa: E501
            return data

    def list_software_environment_analyses_for_build_with_http_info(self, environment_name, **kwargs):  # noqa: E501
        """List analyses for the given software environment for build.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environment_analyses_for_build_with_http_info(environment_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str environment_name: Software environment name for run for which analyses should be retrieved.  (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['environment_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_software_environment_analyses_for_build" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'environment_name' is set
        if ('environment_name' not in params or
                params['environment_name'] is None):
            raise ValueError("Missing the required parameter `environment_name` when calling `list_software_environment_analyses_for_build`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'environment_name' in params:
            path_params['environment_name'] = params['environment_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/build-software-environment/analyses/{environment_name}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_software_environment_analyses_for_run(self, environment_name, **kwargs):  # noqa: E501
        """List analyses for the given software environment for run.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environment_analyses_for_run(environment_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str environment_name: Software environment name for run for which analyses should be retrieved.  (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_software_environment_analyses_for_run_with_http_info(environment_name, **kwargs)  # noqa: E501
        else:
            (data) = self.list_software_environment_analyses_for_run_with_http_info(environment_name, **kwargs)  # noqa: E501
            return data

    def list_software_environment_analyses_for_run_with_http_info(self, environment_name, **kwargs):  # noqa: E501
        """List analyses for the given software environment for run.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environment_analyses_for_run_with_http_info(environment_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str environment_name: Software environment name for run for which analyses should be retrieved.  (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['environment_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_software_environment_analyses_for_run" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'environment_name' is set
        if ('environment_name' not in params or
                params['environment_name'] is None):
            raise ValueError("Missing the required parameter `environment_name` when calling `list_software_environment_analyses_for_run`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'environment_name' in params:
            path_params['environment_name'] = params['environment_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/run-software-environment/analyses/{environment_name}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_software_environments_for_build(self, **kwargs):  # noqa: E501
        """Retrieve a list of software environments analyzed for build.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environments_for_build(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int page: Page offset in pagination.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_software_environments_for_build_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.list_software_environments_for_build_with_http_info(**kwargs)  # noqa: E501
            return data

    def list_software_environments_for_build_with_http_info(self, **kwargs):  # noqa: E501
        """Retrieve a list of software environments analyzed for build.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environments_for_build_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int page: Page offset in pagination.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['page']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_software_environments_for_build" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/build-software-environment', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_software_environments_for_run(self, **kwargs):  # noqa: E501
        """Retrieve a list of software environments analyzed for run.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environments_for_run(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int page: Page offset in pagination.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_software_environments_for_run_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.list_software_environments_for_run_with_http_info(**kwargs)  # noqa: E501
            return data

    def list_software_environments_for_run_with_http_info(self, **kwargs):  # noqa: E501
        """Retrieve a list of software environments analyzed for run.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_software_environments_for_run_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int page: Page offset in pagination.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['page']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_software_environments_for_run" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/run-software-environment', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def post_analyze(self, image, **kwargs):  # noqa: E501
        """Analyze the given image asynchronously.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_analyze(image, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image: Name of image - can also specify remote registry to pull image from.  (required)
        :param str registry_user: Registry user to be used for pulling images from registry. 
        :param str registry_password: Registry password or token to be used for pulling images from registry. 
        :param str environment_type: Type of environment (runtime or buildtime) which is being analyzed. 
        :param str origin: A remote where the image is being used. This is used for tracking as well as for automated reporting when results are available. 
        :param bool debug: Run the given analyzer in a verbose mode so developers can debug analyzer. 
        :param bool verify_tls: Verify TLS certificates of registry from where images are pulled from. 
        :param bool force: Do not use cached results, always run analysis. 
        :return: AnalysisResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_analyze_with_http_info(image, **kwargs)  # noqa: E501
        else:
            (data) = self.post_analyze_with_http_info(image, **kwargs)  # noqa: E501
            return data

    def post_analyze_with_http_info(self, image, **kwargs):  # noqa: E501
        """Analyze the given image asynchronously.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_analyze_with_http_info(image, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image: Name of image - can also specify remote registry to pull image from.  (required)
        :param str registry_user: Registry user to be used for pulling images from registry. 
        :param str registry_password: Registry password or token to be used for pulling images from registry. 
        :param str environment_type: Type of environment (runtime or buildtime) which is being analyzed. 
        :param str origin: A remote where the image is being used. This is used for tracking as well as for automated reporting when results are available. 
        :param bool debug: Run the given analyzer in a verbose mode so developers can debug analyzer. 
        :param bool verify_tls: Verify TLS certificates of registry from where images are pulled from. 
        :param bool force: Do not use cached results, always run analysis. 
        :return: AnalysisResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['image', 'registry_user', 'registry_password', 'environment_type', 'origin', 'debug', 'verify_tls', 'force']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_analyze" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'image' is set
        if ('image' not in params or
                params['image'] is None):
            raise ValueError("Missing the required parameter `image` when calling `post_analyze`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'image' in params:
            query_params.append(('image', params['image']))  # noqa: E501
        if 'registry_user' in params:
            query_params.append(('registry_user', params['registry_user']))  # noqa: E501
        if 'registry_password' in params:
            query_params.append(('registry_password', params['registry_password']))  # noqa: E501
        if 'environment_type' in params:
            query_params.append(('environment_type', params['environment_type']))  # noqa: E501
        if 'origin' in params:
            query_params.append(('origin', params['origin']))  # noqa: E501
        if 'debug' in params:
            query_params.append(('debug', params['debug']))  # noqa: E501
        if 'verify_tls' in params:
            query_params.append(('verify_tls', params['verify_tls']))  # noqa: E501
        if 'force' in params:
            query_params.append(('force', params['force']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/analyze', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AnalysisResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def post_image_metadata(self, image, **kwargs):  # noqa: E501
        """Get metadata for the given image  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_image_metadata(image, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image: Name of image - can also specify remote registry to pull image from.  (required)
        :param str registry_user: Registry user to be used for pulling images from registry. 
        :param str registry_password: Registry password or token to be used for pulling images from registry. 
        :param bool verify_tls: Verify TLS certificates of registry from where images are pulled from. 
        :return: ImageMetadataResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_image_metadata_with_http_info(image, **kwargs)  # noqa: E501
        else:
            (data) = self.post_image_metadata_with_http_info(image, **kwargs)  # noqa: E501
            return data

    def post_image_metadata_with_http_info(self, image, **kwargs):  # noqa: E501
        """Get metadata for the given image  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_image_metadata_with_http_info(image, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image: Name of image - can also specify remote registry to pull image from.  (required)
        :param str registry_user: Registry user to be used for pulling images from registry. 
        :param str registry_password: Registry password or token to be used for pulling images from registry. 
        :param bool verify_tls: Verify TLS certificates of registry from where images are pulled from. 
        :return: ImageMetadataResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['image', 'registry_user', 'registry_password', 'verify_tls']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_image_metadata" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'image' is set
        if ('image' not in params or
                params['image'] is None):
            raise ValueError("Missing the required parameter `image` when calling `post_image_metadata`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'image' in params:
            query_params.append(('image', params['image']))  # noqa: E501
        if 'registry_user' in params:
            query_params.append(('registry_user', params['registry_user']))  # noqa: E501
        if 'registry_password' in params:
            query_params.append(('registry_password', params['registry_password']))  # noqa: E501
        if 'verify_tls' in params:
            query_params.append(('verify_tls', params['verify_tls']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/image/metadata', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ImageMetadataResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class RuntimeEnvironment(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cuda_version': 'str',
        'hardware': 'object',
        'ipython': 'object',
        'name': 'str',
        'operating_system': 'object',
        'platform': 'str',
        'python_version': 'str'
    }

    attribute_map = {
        'cuda_version': 'cuda_version',
        'hardware': 'hardware',
        'ipython': 'ipython',
        'name': 'name',
        'operating_system': 'operating_system',
        'platform': 'platform',
        'python_version': 'python_version'
    }

    def __init__(self, cuda_version=None, hardware=None, ipython=None, name=None, operating_system=None, platform=None, python_version=None):  # noqa: E501
        """RuntimeEnvironment - a model defined in Swagger"""  # noqa: E501
        self._cuda_version = None
        self._hardware = None
        self._ipython = None
        self._name = None
        self._operating_system = None
        self._platform = None
        self._python_version = None
        self.discriminator = None
        if cuda_version is not None:
            self.cuda_version = cuda_version
        if hardware is not None:
            self.hardware = hardware
        if ipython is not None:
            self.ipython = ipython
        if name is not None:
            self.name = name
        if operating_system is not None:
            self.operating_system = operating_system
        if platform is not None:
            self.platform = platform
        if python_version is not None:
            self.python_version = python_version

    @property
    def cuda_version(self):
        """Gets the cuda_version of this RuntimeEnvironment.  # noqa: E501

        Nvidia CUDA version which the application uses.  # noqa: E501

        :return: The cuda_version of this RuntimeEnvironment.  # noqa: E501
        :rtype: str
        """
        return self._cuda_version

    @cuda_version.setter
    def cuda_version(self, cuda_version):
        """Sets the cuda_version of this RuntimeEnvironment.

        Nvidia CUDA version which the application uses.  # noqa: E501

        :param cuda_version: The cuda_version of this RuntimeEnvironment.  # noqa: E501
        :type: str
        """

        self._cuda_version = cuda_version

    @property
    def hardware(self):
        """Gets the hardware of this RuntimeEnvironment.  # noqa: E501

        Hardware configuration used.  # noqa: E501

        :return: The hardware of this RuntimeEnvironment.  # noqa: E501
        :rtype: object
        """
        return self._hardware

    @hardware.setter
    def hardware(self, hardware):
        """Sets the hardware of this RuntimeEnvironment.

        Hardware configuration used.  # noqa: E501

        :param hardware: The hardware of this RuntimeEnvironment.  # noqa: E501
        :type: object
        """

        self._hardware = hardware

    @property
    def ipython(self):
        """Gets the ipython of this RuntimeEnvironment.  # noqa: E501

        Additional information captured when recommendations were requested for Jupyter Notebook environments.   # noqa: E501

        :return: The ipython of this RuntimeEnvironment.  # noqa: E501
        :rtype: object
        """
        return self._ipython

    @ipython.setter
    def ipython(self, ipython):
        """Sets the ipython of this RuntimeEnvironment.

        Additional information captured when recommendations were requested for Jupyter Notebook environments.   # noqa: E501

        :param ipython: The ipython of this RuntimeEnvironment.  # noqa: E501
        :type: object
        """

        self._ipython = ipython

    @property
    def name(self):
        """Gets the name of this RuntimeEnvironment.  # noqa: E501

        User defined name of the runtime environment.  # noqa: E501

        :return: The name of this RuntimeEnvironment.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this RuntimeEnvironment.

        User defined name of the runtime environment.  # noqa: E501

        :param name: The name of this RuntimeEnvironment.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def operating_system(self):
        """Gets the operating_system of this RuntimeEnvironment.  # noqa: E501

        Operating system used.  # noqa: E501

        :return: The operating_system of this RuntimeEnvironment.  # noqa: E501
        :rtype: object
        """
        return self._operating_system

    @operating_system.setter
    def operating_system(self, operating_system):
        """Sets the operating_system of this RuntimeEnvironment.

        Operating system used.  # noqa: E501

        :param operating_system: The operating_system of this RuntimeEnvironment.  # noqa: E501
        :type: object
        """

        self._operating_system = operating_system

    @property
    def platform(self):
        """Gets the platform of this RuntimeEnvironment.  # noqa: E501

        Platform used - corresponds to sysconfig.get_platform() call.  # noqa: E501

        :return: The platform of this RuntimeEnvironment.  # noqa: E501
        :rtype: str
        """
        return self._platform

    @platform.setter
    def platform(self, platform):
        """Sets the platform of this RuntimeEnvironment.

        Platform used - corresponds to sysconfig.get_platform() call.  # noqa: E501

        :param platform: The platform of this RuntimeEnvironment.  # noqa: E501
        :type: str
        """

        self._platform = platform

    @property
    def python_version(self):
        """Gets the python_version of this RuntimeEnvironment.  # noqa: E501

        Python version on which the application runs on.  # noqa: E501

        :return: The python_version of this RuntimeEnvironment.  # noqa: E501
        :rtype: str
        """
        return self._python_version

    @python_version.setter
    def python_version(self, python_version):
        """Sets the python_version of this RuntimeEnvironment.

        Python version on which the application runs on.  # noqa: E501

        :param python_version: The python_version of this RuntimeEnvironment.  # noqa: E501
        :type: str
        """

        self._python_version = python_version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RuntimeEnvironment, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RuntimeEnvironment):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class PythonPackageMetadataResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'author': 'str',
        'author_email': 'str',
        'classifier': 'str',
        'description_content_type': 'str',
        'download_url': 'str',
        'home_page': 'str',
        'keywords': 'str',
        'license': 'str',
        'maintainer': 'str',
        'maintainer_email': 'str',
        'metadata_version': 'str',
        'name': 'str',
        'platform': 'str',
        'project_url': 'str',
        'provides_extra': 'str',
        'requires_dist': 'str',
        'requires_python': 'str',
        'summary': 'str',
        'version': 'str'
    }

    attribute_map = {
        'author': 'author',
        'author_email': 'author_email',
        'classifier': 'classifier',
        'description_content_type': 'description_content_type',
        'download_url': 'download_url',
        'home_page': 'home_page',
        'keywords': 'keywords',
        'license': 'license',
        'maintainer': 'maintainer',
        'maintainer_email': 'maintainer_email',
        'metadata_version': 'metadata_version',
        'name': 'name',
        'platform': 'platform',
        'project_url': 'project_url',
        'provides_extra': 'provides_extra',
        'requires_dist': 'requires_dist',
        'requires_python': 'requires_python',
        'summary': 'summary',
        'version': 'version'
    }

    def __init__(self, author=None, author_email=None, classifier=None, description_content_type=None, download_url=None, home_page=None, keywords=None, license=None, maintainer=None, maintainer_email=None, metadata_version=None, name=None, platform=None, project_url=None, provides_extra=None, requires_dist=None, requires_python=None, summary=None, version=None):  # noqa: E501
        """PythonPackageMetadataResponse - a model defined in Swagger"""  # noqa: E501
        self._author = None
        self._author_email = None
        self._classifier = None
        self._description_content_type = None
        self._download_url = None
        self._home_page = None
        self._keywords = None
        self._license = None
        self._maintainer = None
        self._maintainer_email = None
        self._metadata_version = None
        self._name = None
        self._platform = None
        self._project_url = None
        self._provides_extra = None
        self._requires_dist = None
        self._requires_python = None
        self._summary = None
        self._version = None
        self.discriminator = None
        self.author = author
        self.author_email = author_email
        self.classifier = classifier
        self.description_content_type = description_content_type
        self.download_url = download_url
        self.home_page = home_page
        self.keywords = keywords
        self.license = license
        self.maintainer = maintainer
        self.maintainer_email = maintainer_email
        self.metadata_version = metadata_version
        self.name = name
        self.platform = platform
        self.project_url = project_url
        self.provides_extra = provides_extra
        self.requires_dist = requires_dist
        self.requires_python = requires_python
        self.summary = summary
        self.version = version

    @property
    def author(self):
        """Gets the author of this PythonPackageMetadataResponse.  # noqa: E501

        A string containing the author’s name.  # noqa: E501

        :return: The author of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._author

    @author.setter
    def author(self, author):
        """Sets the author of this PythonPackageMetadataResponse.

        A string containing the author’s name.  # noqa: E501

        :param author: The author of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if author is None:
            raise ValueError("Invalid value for `author`, must not be `None`")  # noqa: E501

        self._author = author

    @property
    def author_email(self):
        """Gets the author_email of this PythonPackageMetadataResponse.  # noqa: E501

        A string containing the author’s e-mail address.  # noqa: E501

        :return: The author_email of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._author_email

    @author_email.setter
    def author_email(self, author_email):
        """Sets the author_email of this PythonPackageMetadataResponse.

        A string containing the author’s e-mail address.  # noqa: E501

        :param author_email: The author_email of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if author_email is None:
            raise ValueError("Invalid value for `author_email`, must not be `None`")  # noqa: E501

        self._author_email = author_email

    @property
    def classifier(self):
        """Gets the classifier of this PythonPackageMetadataResponse.  # noqa: E501

        Each entry is a string giving a single classification value for the distribution.  # noqa: E501

        :return: The classifier of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._classifier

    @classifier.setter
    def classifier(self, classifier):
        """Sets the classifier of this PythonPackageMetadataResponse.

        Each entry is a string giving a single classification value for the distribution.  # noqa: E501

        :param classifier: The classifier of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if classifier is None:
            raise ValueError("Invalid value for `classifier`, must not be `None`")  # noqa: E501

        self._classifier = classifier

    @property
    def description_content_type(self):
        """Gets the description_content_type of this PythonPackageMetadataResponse.  # noqa: E501

        Content-Type of description text.  # noqa: E501

        :return: The description_content_type of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._description_content_type

    @description_content_type.setter
    def description_content_type(self, description_content_type):
        """Sets the description_content_type of this PythonPackageMetadataResponse.

        Content-Type of description text.  # noqa: E501

        :param description_content_type: The description_content_type of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if description_content_type is None:
            raise ValueError("Invalid value for `description_content_type`, must not be `None`")  # noqa: E501

        self._description_content_type = description_content_type

    @property
    def download_url(self):
        """Gets the download_url of this PythonPackageMetadataResponse.  # noqa: E501

        A string containing the URL from which this version of the distribution can be downloaded.  # noqa: E501

        :return: The download_url of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._download_url

    @download_url.setter
    def download_url(self, download_url):
        """Sets the download_url of this PythonPackageMetadataResponse.

        A string containing the URL from which this version of the distribution can be downloaded.  # noqa: E501

        :param download_url: The download_url of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if download_url is None:
            raise ValueError("Invalid value for `download_url`, must not be `None`")  # noqa: E501

        self._download_url = download_url

    @property
    def home_page(self):
        """Gets the home_page of this PythonPackageMetadataResponse.  # noqa: E501

        A string containing the URL for the distribution’s home page.  # noqa: E501

        :return: The home_page of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._home_page

    @home_page.setter
    def home_page(self, home_page):
        """Sets the home_page of this PythonPackageMetadataResponse.

        A string containing the URL for the distribution’s home page.  # noqa: E501

        :param home_page: The home_page of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if home_page is None:
            raise ValueError("Invalid value for `home_page`, must not be `None`")  # noqa: E501

        self._home_page = home_page

    @property
    def keywords(self):
        """Gets the keywords of this PythonPackageMetadataResponse.  # noqa: E501

        A list of additional keywords to be used to assist searching for the distribution in a larger catalog.   # noqa: E501

        :return: The keywords of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._keywords

    @keywords.setter
    def keywords(self, keywords):
        """Sets the keywords of this PythonPackageMetadataResponse.

        A list of additional keywords to be used to assist searching for the distribution in a larger catalog.   # noqa: E501

        :param keywords: The keywords of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if keywords is None:
            raise ValueError("Invalid value for `keywords`, must not be `None`")  # noqa: E501

        self._keywords = keywords

    @property
    def license(self):
        """Gets the license of this PythonPackageMetadataResponse.  # noqa: E501

        Text indicating the license covering the distribution.  # noqa: E501

        :return: The license of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._license

    @license.setter
    def license(self, license):
        """Sets the license of this PythonPackageMetadataResponse.

        Text indicating the license covering the distribution.  # noqa: E501

        :param license: The license of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if license is None:
            raise ValueError("Invalid value for `license`, must not be `None`")  # noqa: E501

        self._license = license

    @property
    def maintainer(self):
        """Gets the maintainer of this PythonPackageMetadataResponse.  # noqa: E501

        A string containing the maintainer’s name at a minimum; additional contact information may be provided.   # noqa: E501

        :return: The maintainer of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._maintainer

    @maintainer.setter
    def maintainer(self, maintainer):
        """Sets the maintainer of this PythonPackageMetadataResponse.

        A string containing the maintainer’s name at a minimum; additional contact information may be provided.   # noqa: E501

        :param maintainer: The maintainer of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if maintainer is None:
            raise ValueError("Invalid value for `maintainer`, must not be `None`")  # noqa: E501

        self._maintainer = maintainer

    @property
    def maintainer_email(self):
        """Gets the maintainer_email of this PythonPackageMetadataResponse.  # noqa: E501

        A string containing the maintainer’s e-mail address.  # noqa: E501

        :return: The maintainer_email of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._maintainer_email

    @maintainer_email.setter
    def maintainer_email(self, maintainer_email):
        """Sets the maintainer_email of this PythonPackageMetadataResponse.

        A string containing the maintainer’s e-mail address.  # noqa: E501

        :param maintainer_email: The maintainer_email of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if maintainer_email is None:
            raise ValueError("Invalid value for `maintainer_email`, must not be `None`")  # noqa: E501

        self._maintainer_email = maintainer_email

    @property
    def metadata_version(self):
        """Gets the metadata_version of this PythonPackageMetadataResponse.  # noqa: E501

        Version of the file format.  # noqa: E501

        :return: The metadata_version of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._metadata_version

    @metadata_version.setter
    def metadata_version(self, metadata_version):
        """Sets the metadata_version of this PythonPackageMetadataResponse.

        Version of the file format.  # noqa: E501

        :param metadata_version: The metadata_version of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if metadata_version is None:
            raise ValueError("Invalid value for `metadata_version`, must not be `None`")  # noqa: E501

        self._metadata_version = metadata_version

    @property
    def name(self):
        """Gets the name of this PythonPackageMetadataResponse.  # noqa: E501

        Name of the distribution.  # noqa: E501

        :return: The name of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this PythonPackageMetadataResponse.

        Name of the distribution.  # noqa: E501

        :param name: The name of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def platform(self):
        """Gets the platform of this PythonPackageMetadataResponse.  # noqa: E501

        A Platform specification describing an operating system supported by the distribution.  # noqa: E501

        :return: The platform of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._platform

    @platform.setter
    def platform(self, platform):
        """Sets the platform of this PythonPackageMetadataResponse.

        A Platform specification describing an operating system supported by the distribution.  # noqa: E501

        :param platform: The platform of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if platform is None:
            raise ValueError("Invalid value for `platform`, must not be `None`")  # noqa: E501

        self._platform = platform

    @property
    def project_url(self):
        """Gets the project_url of this PythonPackageMetadataResponse.  # noqa: E501

        URL to project.  # noqa: E501

        :return: The project_url of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._project_url

    @project_url.setter
    def project_url(self, project_url):
        """Sets the project_url of this PythonPackageMetadataResponse.

        URL to project.  # noqa: E501

        :param project_url: The project_url of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if project_url is None:
            raise ValueError("Invalid value for `project_url`, must not be `None`")  # noqa: E501

        self._project_url = project_url

    @property
    def provides_extra(self):
        """Gets the provides_extra of this PythonPackageMetadataResponse.  # noqa: E501

        Provided extra of the distribution.  # noqa: E501

        :return: The provides_extra of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._provides_extra

    @provides_extra.setter
    def provides_extra(self, provides_extra):
        """Sets the provides_extra of this PythonPackageMetadataResponse.

        Provided extra of the distribution.  # noqa: E501

        :param provides_extra: The provides_extra of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if provides_extra is None:
            raise ValueError("Invalid value for `provides_extra`, must not be `None`")  # noqa: E501

        self._provides_extra = provides_extra

    @property
    def requires_dist(self):
        """Gets the requires_dist of this PythonPackageMetadataResponse.  # noqa: E501

        Each entry contains a string naming some other distutils project required by this distribution.   # noqa: E501

        :return: The requires_dist of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._requires_dist

    @requires_dist.setter
    def requires_dist(self, requires_dist):
        """Sets the requires_dist of this PythonPackageMetadataResponse.

        Each entry contains a string naming some other distutils project required by this distribution.   # noqa: E501

        :param requires_dist: The requires_dist of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if requires_dist is None:
            raise ValueError("Invalid value for `requires_dist`, must not be `None`")  # noqa: E501

        self._requires_dist = requires_dist

    @property
    def requires_python(self):
        """Gets the requires_python of this PythonPackageMetadataResponse.  # noqa: E501

        Python requirements for the distribution.  # noqa: E501

        :return: The requires_python of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._requires_python

    @requires_python.setter
    def requires_python(self, requires_python):
        """Sets the requires_python of this PythonPackageMetadataResponse.

        Python requirements for the distribution.  # noqa: E501

        :param requires_python: The requires_python of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if requires_python is None:
            raise ValueError("Invalid value for `requires_python`, must not be `None`")  # noqa: E501

        self._requires_python = requires_python

    @property
    def summary(self):
        """Gets the summary of this PythonPackageMetadataResponse.  # noqa: E501

        A one-line summary of what the distribution does.  # noqa: E501

        :return: The summary of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._summary

    @summary.setter
    def summary(self, summary):
        """Sets the summary of this PythonPackageMetadataResponse.

        A one-line summary of what the distribution does.  # noqa: E501

        :param summary: The summary of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if summary is None:
            raise ValueError("Invalid value for `summary`, must not be `None`")  # noqa: E501

        self._summary = summary

    @property
    def version(self):
        """Gets the version of this PythonPackageMetadataResponse.  # noqa: E501

        version of the distribution.  # noqa: E501

        :return: The version of this PythonPackageMetadataResponse.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this PythonPackageMetadataResponse.

        version of the distribution.  # noqa: E501

        :param version: The version of this PythonPackageMetadataResponse.  # noqa: E501
        :type: str
        """
        if version is None:
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PythonPackageMetadataResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PythonPackageMetadataResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
